#! /bin/bash

# Set a variable as the absolute path to an executable. The alternative
# would be to make sure 'blastn' is in the shell's PATH.

BLASTN=/usr/local/packages/bioinformatics/ncbiblast/2.2.28/gcc-4.4.6/bin/blastn

# BLASTN uses OpenMP, so we'll set up 1 thread per core allowed for workers.

export OMP_NUM_THREADS=${WQ_CPT}

# Treat the first command line argument as the input file pathname.

FILE=$1
DIR=`dirname ${FILE}`
BASE=`basename ${FILE}`

# Build the rather complex command line, including numactl string. 

CMD="${WQ_NUMACTL} ${BLASTN} -task blastn -outfmt 7 -max_target_seqs 1"
CMD="${CMD} -num_threads ${OMP_NUM_THREADS}"
CMD="${CMD} -db /project/db/img_v400_custom/img_v400_custom_GENOME"
CMD="${CMD} -query ${FILE}"
CMD="${CMD} -out ${DIR}/IMG_genome_blast.${BASE}"

# For testing purposes, use "if false". For real runs, use "if true":
# The random times simulate a spread in task times.

if false ; then
   eval "${CMD}"
else
   echo "Would have executed the following command:"
   echo "${CMD}"
   T=`expr 2 + $RANDOM % 10`
   echo "Sleeping for $T seconds."
   sleep $T
fi
