# Main support stuff for WQ input generator.
#
# Establish global variables.

global tk_strictMotif ;  # Flag to indicate type Motif-style dialog boxes.

global wqDefault ;       # Index of default value.
global wqDepends ;       # Index of dependent variable name.
global wqEditKeyword ;   # Keyword selected.
global wqEditLegal ;     # Legal values for selected keyword.
global wqEditNewValue ;  # Desired new value of keyword.
global wqEditOldValue ;  # Current value of keyword.
global wqEditParam ;     # Index of selected keyword in wqSettingListbox.
global wqFile ;          # File name string.
global wqHosts ;         # List of host specific settings.
global wqHostsLegal ;    # List of legal host names.
global wqKey ;           # Key word definitions.
global wqKeywords ;      # Associative array of keyword settings.
global wqKeywordsLegal ; # List of legal keywords.
global wqModified ;      # Flag to indicate if keywords have been modified.
global wqNumKeywords ;   # Number of keywords with active values.
global wqOpt ;           # Index of option type.
global wqPPN ;           # Index for PPN value.
global wqQueue ;         # Index for Queue value.
global wqType ;          # Type of parameter.
global wqOptional ;      # Index for optional types.
global wqRequired ;      # Index for required types.
global wqUnused ;	 # Flag indicating keyword is not set.
global wqUsage ;         # Index of usage comment.
global wqValues ;        # Index of legal value string.

# Initialize some of the variables.

set tk_strictMotif 1
set wqEditParam -1
set wqFile "(No File)"
set wqHosts(mike) [list 16 "workq checkpt"]
set wqHosts(tezpur) [list 4 "workq checkpt"]
set wqHostsLegal [list mike tezpur]
set wqModified "No Change"
set wqNumKeywords "(No Keywords)"
set wqPPN 0
set wqQueue 1
set wqUnused "(not specified)"

set wqType [list Optional Required]
set wqOptional 0
set wqRequired 1

# The order here sets the order of appearance in the listbox. 
#
# Determined from host name:  PPN, available queue names
# Determined from job name: joboutput, jobscript, filelist, taskscript
# Preset jobtypes:  serial, mpi, openmp
# workerspernode can be computed.
# Name of joboutput file can be computed.

set wqKeywordsLegal [list jobname fileext allocation hostname nodes \
			 walltime queue workdir tasktype \
			 procspertask cmdline ]

# For each keyword, set the range of values, option status, and usage text.
# Numerical tests are of the form:  Type Test Value
# These are then used in the sanity checks to make values are valid.
# The usage strings should be less than 38 characters per line.

set wqValues 0
set wqOpt 1
set wqUsage 2
set wqDefault 3
set wqDepends 4

set wqKey(jobname) [list "Any: string" 0 "Job name to assign" WQ_Job none]
set wqKey(fileext) [list "Any: string" 0 \
			"Input files have this extension" .dat none]
set wqKey(allocation) [list "Any: string" 1 \
		       "Allocation code to charge against" none none]
set wqKey(hostname) [list "Choice: $wqHostsLegal" 1 \
		     "Name of host running job" mike none]
set wqKey(nodes) [list "Int: > 0" 0 \
		  "Number of nodes to request" 1 none]
set wqKey(walltime) [list "Time: hh:mm:ss" 0 \
		     "Total wallclock time for job" 01:00:00]
set wqKey(queue) [list "Choice: checkpt workq" 0 \
		  "Queue to submit job to" workq hostname]
set wqKey(workdir) [list "Any: /path/to/work/dir" 1 \
		    "Absolute path to work directory" none none]
set wqKey(tasktype) [list "Choice: serial mpi openmp" 0 \
		     "Multi-process characteristic of tasks" serial none]
set wqKey(procspertask) [list "Int: > 0" 0 \
			 "Number of processes per task" 1 none]
set wqKey(cmdline) [list "Any: string" 1 \
		    "Command line for task" none none]

# Set up event bindings.

bind .wqSettingListbox <ButtonRelease-1> {
    wqUpdateParam [.wqSettingListbox curselection]
}

bind .wqKeywordListbox <ButtonRelease-1> {
    wqUpdateParam [.wqKeywordListbox curselection]
}

bind .wqEditNewValue <Return> {
    wqAcceptNewParam
}

# Define the File menu button.

menu .wqFileMenuButton.m

.wqFileMenuButton.m add command -label "New" -command { wqNew } \
	-accelerator "Alt-n"
.wqFileMenuButton.m add command -label "Open .." -command { wqOpen } \
	-accelerator "Alt-o"
.wqFileMenuButton.m add command -label "Save .." -command { wqSave } \
	-accelerator "Alt-s"
.wqFileMenuButton.m add separator
.wqFileMenuButton.m add command -label "Revert" -command { wqRevert } \
	-accelerator "Alt-r"
.wqFileMenuButton.m add separator
.wqFileMenuButton.m add command -label "Generate" -command { wqGenerate } \
        -accelerator "Alt-g"
.wqFileMenuButton.m add separator
.wqFileMenuButton.m add command -label "Quit" -command { wqQuit } \
	-accelerator "Alt-q"

bind . <Alt-Key-n> { wqNew }
bind . <Alt-Key-o> { wqOpen }
bind . <Alt-Key-s> { wqSave }
bind . <Alt-Key-r> { wqRevert }
bind . <Alt-Key-q> { wqQuit }
bind . <Alt-Key-g> { wqGenerate }


# Procedure to handle the New command

proc wqNew { } {

    global wqFile
    global wqNumKeywords
    global wqModified

    if { [string compare $wqModified "Modified"] == 0 } {
	set ans [tk_messageBox -icon question -title "FILE MODIFIED" \
		 -type yesno -message "Save before starting new?"]
	if { [string compare $ans "yes"] == 0 } {
	    wqSave
	}
    }

    set wqNumKeywords 0
    set wqFile "(none)"
    wqKeywords_Init
    wqLoadListboxes
}


# Procedure to handle the Open command

proc wqOpen { } {

    global wqNumKeywords
    global wqKeywords
    global wqFile
    global wqModified

    if { [string compare $wqModified "Modified"] == 0 } {
	set ans [tk_messageBox -icon question -title "FILE MODIFIED" \
		 -type yesno -message "Save before opening?"]
	if { [string compare $ans "yes"] == 0 } {
	    wqSave
	}
    }

    set wqFile [tk_getOpenFile]
    if { [string length "$wqFile"] > 0 } {
	wqKeywordsRead $wqFile
	wqLoadListboxes
    }
}


# Procedure to handle the Save command

proc wqSave { } {

    global wqModified
    global wqFile

    if { [string compare $wqModified "Modified"] == 0 } {
	if [wqKeywordCheck] {
	    set wqFile [tk_getSaveFile]
	    if { [string length "$wqFile"] > 0 } {
		wqKeywordsWrite $wqFile
		set wqModified "No Change"
	    }
	}
    }
}


# Do some sanity checking on keyword settings.  

proc wqParamCheck { k v } {
    # k is the keyword to check.
    # v is the proposed value to assign.

    global wqKey
    global wqValues

    set returnvalue 1
    set key $k
    set kvs [lindex $wqKey($k) $wqValues]
    switch [lindex $kvs 0] {
	Any: { 
	    if { [llength $v] < 1 } {
		set returnvalue 0
		tk_messageBox -type ok -icon error -title "INCORRECT SETTING" \
		    -message "$key must be a string."
	    }
	}
	Choice: {
	    set legal [lrange $kvs 1 end]
	    if { [lsearch -exact $legal $v] == -1 } {
		set returnvalue 0
		tk_messageBox -type ok -icon error -title "ILLEGAL VALUE" \
		    -message "$key must be one of:
$legal!" }
	}
	Int: {
	    set test [lindex $kvs 1]
	    set tv [lindex $kvs 2]
	    if { [regexp -all {[0-9]} $v] != [string length $v] } {
		set returnvalue 0
		tk_messageBox -type ok -icon error -title "ILLEGAL VALUE" \
		    -message "$key must be Integer."
	    } else {
		set ok 0
		switch $test {
		    ">" { if { $v > $tv } { set ok 1 } }
		    default { }
		}
		if { $ok == 0 } { 
		    set returnvalue 0
		    tk_messageBox -type ok -icon error -title "ILLEGAL VALUE" \
			-message "$key must be $test $tv"
		}
	    }
	}
	Time: { 
	    if { [regexp -all {[0-9][0-9]:[0-9][0-9]:[0-9][0-9]} $v] != 1 } {
		set returnvalue 0
		tk_messageBox -type ok -icon error -title "ILLEGAL TIME" \
		    -message "$key must be hh:mm:ss"
	    }
	}
    }
    return $returnvalue
}


proc wqKeywordCheck { } {

    global wqKey
    global wqOpt
    global wqUnused
    global wqType
    global wqRequired

    set returnvalue 1
    set missing {}

    # First, build temp arrays of keywords and values.  Check for missing
    # required keywords, then for legal values.

    set last [.wqKeywordListbox index end]
    incr last -1

    for {set p 0} {$p <= $last} {incr p} {
	set k [string tolower [.wqKeywordListbox get $p]]
	set v($k) [.wqSettingListbox get $p]
	set opt [lindex $wqType [lindex $wqKey($k) $wqOpt]]
	if { [string compare $v($k) $wqUnused] == 0 && \
		 [lindex $wqKey($k) $wqOpt] == $wqRequired } {
	    lappend missing $k
        } else {
	    if { [wqParamCheck $k $v($k)] == 0 } {set returnvalue 0}
	}
    }

    if { [llength $missing] != 0 } {

	 tk_messageBox -type ok -icon error \
		 -title "MISSING REQUIRED KEYWORDS" \
		 -message "$missing"

	 set returnvalue 0
    }

    return $returnvalue
}


# Check if a keyword mandated by another keyword has been specified.

proc wqRequiredPair { a av b bv } {

    global wqKey
    global wqUnused

    if { [string compare $av $wqUnused] != 0 && \
	     [string compare $bv $wqUnused] == 0 } {
	tk_messageBox -type ok -icon error \
	    -title "UNSET KEYWORD" \
	    -message "$a REQUIRES $b"
	return 0
    }

    return 1
}


# Revert to original file settings

proc wqRevert { } {
    
    set answer [tk_messageBox -title "LAST CHANCE" -type yesno -icon question \
	    -message "This will reset all values to
those originally loaded.
Are you sure?"]
    if { [string compare $answer yes] } {
	wqLoadListboxes
    }
}


# Procedure to handle the Generate command

proc wqGenerate { } {

    global wqModified
    global wqFile

    if { [string compare $wqModified "Modified"] == 0 } {
	set ans [tk_messageBox -icon question -title "FILE MODIFIED" \
		 -type yesno -message "Save before generating?"]
	if { [string compare $ans "yes"] == 0 } {
	    wqSave
	}
    }
   tk_messageBox -type ok -icon error -title "Not Yet Implemented" \
       -message "Generate function not yet implemented."
}


# Procedure to handle the Quit Command

proc wqQuit { } {

    global wqModified
    global wqFile

    if { [string compare $wqModified "Modified"] == 0 } {
	set ans [tk_messageBox -icon question -title "FILE MODIFIED" \
		 -type yesno -message "Quit without saving changes?"]
	if { [string compare $ans "no"] == 0 } {
	    wqSave
	}
    }
    exit
}


# Procedure to save current settings to a file.

proc wqKeywordsWrite { File } {

    global wqKeywords
    global wqUnused

    if [catch {open $File w 0644} stream] {
	tk_messageBox -title "FILE OPEN FAILURE" -icon error -type ok \
		-message "Failed to open \"$File\""
    } else {
	for {set p 0} {$p <=[.wqKeywordListbox index end]} {incr p} {
	    set k [.wqKeywordListbox get $p]
	    set v [.wqSettingListbox get $p]

	    if { [string compare $v $wqUnused] != 0 } {
		puts $stream "$k = $v"
	    }
	}
    }
    close $stream
}


# Procedure to load the list boxes

proc wqLoadListboxes { } {

    global wqKey
    global wqKeywordsLegal
    global wqKeywords
    global wqModified

    .wqKeywordListbox delete 0 end
    .wqSettingListbox delete 0 end

    foreach p $wqKeywordsLegal {
	.wqKeywordListbox insert end $p
        .wqSettingListbox insert end $wqKeywords($p)
    }

    .wqSettingListbox selection set 0
    wqUpdateParam 0
    set wqModified "No Change"
}


# Procedure to initialize the keywords.

proc wqKeywords_Init { } {

    global wqKey
    global wqOpt
    global wqDefault
    global wqKeywords
    global wqKeywordsLegal
    global wqUnused
    global wqOptional

    foreach p "$wqKeywordsLegal" {
	if { [lindex $wqKey($p) $wqOpt] == $wqOptional } {
	    set wqKeywords($p) [lindex $wqKey($p) $wqDefault]
        } else {
	    set wqKeywords($p) $wqUnused
	}
    }

    set wqEditParam -1
}


# Procedure to link-scroll the Keyword and Setting ListBoxes in unison.

proc wqLinkListboxes { a b {c ""} } {

    if { [string length $c] > 0 } {
	.wqSettingListbox yview $a $b $c
	.wqKeywordListbox yview $a $b $c
    } else {
	.wqSettingListbox yview $a $b
	.wqKeywordListbox yview $a $b
    }
}


# Procedure to parse an WQ input file.

proc wqKeywordsRead { File } {

    global wqNumKeywords
    global wqKeywords
    global wqKeywordsLegal

    # Issue a File Open dialog to get input file name, stored in "File"

    # Now open the file, using "stream" as the file handle.

    if [catch {open $File r} stream] {
	tk_messageBox -title "FILE OPEN FAILURE" -icon error -type ok \
		-message "\"$File\""
	return
    }

    # Read everything in, splitting out individual lines.  Then scan each
    # line and build a temporary list with the keyword and its value.
    # Ignore blank lines and comments (lines beginning with "#").

    wqKeywords_Init

    set wqNumKeywords 0

    foreach line [split [read $stream] \n] {
	set value {}
	set keyword ""

	if { [string length "$line"] != 0 } {
	    foreach f [split $line] {
		if { [string length "$keyword"] == 0 } {
		    set keyword [string tolower $f]
		} else {
		    if { [string length "$f"] != 0
			 && [string compare "$f" "="] != 0 } {
			lappend value "$f"
		    }
		}
	    }
	}

	if { [string length "$keyword"] > 0
	     && [string first "#" $keyword] != 0 } {

		 if { [lsearch -exact "$wqKeywordsLegal" $keyword] != -1 } {
		     set wqKeywords($keyword) "$value"
		     incr wqNumKeywords
		 } else {
		     tk_messageBox -title "UNKNOWN KEYWORD" -icon error \
			     -type ok -message "Culprit:  \"$keyword\""
		 }
	}
    }

    close $stream
}


# Procedure to select a keyword for editing.

proc wqUpdateParam { i } {

    global wqKey
    global wqOpt
    global wqDefault
    global wqValues
    global wqUsage
    global wqType
    global wqEditParam
    global wqEditKeyword
    global wqEditOldValue
    global wqEditNewValue
    global wqEditOption
    global wqEditLegal

    set wqEditParam $i
    set wqEditKeyword [.wqKeywordListbox get $wqEditParam]
    set key [string tolower $wqEditKeyword]
    set wqEditOldValue [.wqSettingListbox get $wqEditParam]
    set wqEditOption [lindex $wqType [lindex $wqKey($key) $wqOpt]]
    set wqEditNewValue ""
    set wqEditLegal [lindex $wqKey($key) $wqValues]

    .wqEditUsage delete 0.0 end
    .wqEditUsage insert 0.0 [lindex $wqKey($key) $wqUsage]
    .wqEditUsage insert end \
	"  (Default: [lindex $wqKey($key) $wqDefault])"
}


# Procedure to reset a keyword to the newly entered value.

proc wqAcceptNewParam { } {

    global wqEditKeyword
    global wqEditNewValue
    global wqEditOldValue
    global wqEditParam
    global wqNumKeywords
    global wqUnused
    global wqModified

    if { $wqEditParam >= 0 } {
	set k [string tolower $wqEditKeyword]
	set t [wqParamCheck $k $wqEditNewValue]
	if { $t != 0 } {
	    if { [string compare $wqUnused $wqEditOldValue] == 0 } {
		if { [string length $wqEditNewValue] == 0 } {
		    set wqEditNewValue $wqUnused
		} else {
		    incr wqNumKeywords
		    set wqModified "Modified"
		}
	    } else {
		if { [string length $wqEditNewValue] == 0 } {
		    set wqEditNewValue $wqUnused
		    incr wqNumKeywords -1
		}
		set wqModified "Modified"
	    }

	    .wqSettingListbox insert $wqEditParam $wqEditNewValue
	    .wqSettingListbox delete [expr $wqEditParam + 1]
	    set wqEditOldValue $wqEditNewValue
	    set wqEditNewValue ""
	    .wqSettingListbox selection set $wqEditParam
	    wqKeywordCheck
	}
    }
}
