# Example Python routines. They have been incorporated in wq.py.

def time2secs( s ):
   """
   Convert time in a string to seconds.

   Args:

      s .. time string. May be 'ss', 'mm:ss', 'hh:mm:ss', 'd:hh:mm:ss'

   Returns:

      Number of seconds as an int.
   """
   t = s.split( ':' )
   nf = len( t )
   if nf == 1:
      # Seconds only!
      secs = int( t[0] )
   elif nf == 2:
      # Minutes & seconds!
      secs = int( t[1] ) + int( t[0] ) * 60
   elif nf == 3:
      # Hours, minutes & seconds!
      secs = int( t[2] ) + int( t[1] ) * 60 + int( t[0] ) * 60 * 60   
   elif nf == 4:
      # Days, hours, minutes, & seconds!
      secs = int( t[3] ) + int( t[2] ) * 60 + int( t[1] ) * 60 * 60
      secs += int( t[0] ) * 60 * 60 * 24

   return secs


def walltime( job ):
   """
   Get expended and requested walltime for a job. Assumes Maui/Torque is
   used for job scheduling as it depends on 'checkjob' to retrieve the
   needed information.

   Args:

      job .. Job identifier in form nnnnn.host

   Returns:

      Doublet of form: [ expended, requested ] 
      The times are in seconds.

   If the walltime check returns an error, the times are set to 0,
   and a warning is written to stderr.

   Note that the expended time is only as accurated as the rate at
   which the data is updated. In other words, not very, at least
   until the job ends.
   """
   # Determine the headnode from job ID (i.e. 78995.mike3 => mike1)

   head = re.search( '[a-z]+', job ).group() + '1'

   # Get the walltime data produced by 'checkjob'

   response = shell( 'ssh %s checkjob %s | grep WallTime' % ( head, job ) )

   if response[0]:
      used = time2secs( response[1][0].split()[1] )
      requested = time2secs( response[1][0].split()[3] )
   else:
      used = 0
      requested = 0
      sys.stderr.write( 'ERROR: "%s" does not appear to be a PBS job ID!\n'
                        % (job) )
   return [ used, requested ]
