#! /bin/bash
#######################################################################
# Begin WQ SLURM support script.
#
# WARNING - translated but not tested!!!!
#
# This section is closely coupled with the behavior and proper usage of
# wq.py. Make sure to use at least the WQ prologue section from the 
# SLURM script distributed with the wq.py you received. It should be
# considered powerful mojo and to be dabbled with at your own peril.
#
# Copyright (C) 2014-2015  James A. Lupo
#
# This program is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Revision: $Id: wq-slurm.sh 254 2016-01-20 21:18:14Z jalupo $
# 
# The script wq.py is expected to be executable and in the jobs PATH.
#
# While not meant to be called directly by the user, this is how the
# SLURM script calls it with the various user settings:
#
#    wq-slurm.sh CMD WPN WORKDIR FILES START TASK VERBOSE JOBNUM
#
#    CMD ...... Is the name of the PBS job script.
#    WPN ...... Number of workers desired per node.
#    WORKDIR .. The working directory to use.
#    FILES .... The name of the file holding the input list.
#    START .... The starting record number in FILES.
#    TASK ..... The path to the task command.
#    VERBOSE .. Set how chatty WQ should be.
#    JOBNUM ... The PBS job number - blank on mother superior,
#               an integer string on compute nodes.    
#
########################################################################

# Check number of arguments, then assign to named variables:

if [[ $# -lt 7 ]]; then
   echo "WQ-SLURM.SH: Wrong number of arguments."
   echo 'Usage:  wq-slurm.sh CMD WPN WORKDIR FILES START TASK VERBOSE JOBNUM'
   echo '        JOBNUM should be blank when called from mother superior.'
   exit 1
fi

JOBFILENM=$1
WPN=$2
WORKDIR=$3
FILES=$4
START=$5
TASK=$6
VERBOSE=$7
JOBNUM=$8

# Set verbosity flag, if desired.

if [ "$VERBOSE" = "1" ] ; then
   VERBOSE=--verbose
else
    VERBOSE=
fi

# Drop into the working directory after making sure it exists.

if [ ! -d ${WORKDIR} ] ; then
   echo "WQ.SLURM Error: WORKDIR = \"${WORKDIR}\" does not exist!"
   exit 1
fi

cd ${WORKDIR}

# Only the mother superior has SLURM_JOB_ID defined, so we will be
# passing it to the other nodes as the only command line argument. Use
# this fact to decide if we are running on the mother superior or a
# compute node:

if [ "${JOBNUM}x" = "x" ] ; then

   # Must be running on the mother superior. Do some basic sanity
   # checking just to be safe.

   if [ ! -r ${FILES} ] ; then
      echo "WQ.SLURM Error: FILES = \"${FILES}\" does not exist or can't be read!"
      exit 1
   fi

   if [ $(wc -l ${FILES} | cut -d ' ' -f 1) -lt 1 ] ; then
      echo "WQ.SLURM Warning: FILES = \"${FILES}\" is empty. No work to do!"
      exit 0
   fi

   if [ ! -x ${TASK} ] ; then
      echo "WQ.SLURM Error: TASK = \"${TASK}\" does not exist or isn't executable!"
      exit 2
   fi

   if [ ${START} -lt 1 ] ; then
      echo "WQ.SLURM Error: START can't be less than 1! Quiting!"
      exit 3
   fi

   # Make sure the dispatcher port is available.
   
   PORT=54321
   P=`netstat -lt | gawk 'BEGIN{x=0;}/'$PORT'/{x=1;}END{print x;}'`
   if [ $P = '1' ] ; then
       echo "WQ.SLURM Error: Port $PORT is busy?!? Quiting!"
       exit 4
   fi 

   # Remember our host name.

   MS=`uname -n`

   # Use a bit of magic to strip off the trailing host name and
   # leave only the job number from PBS_JOB_ID (machine dependent?):

   JOBNUM=${SLURM_JOB_ID%.*}
   HOSTLIST=${WORKDIR}/hostlist.${JOBNUM}

   # We want the mother superior host name first. So, take the host
   # list provided, sort it into a unique list of names, with MS first.
   # This assures it's node ID, or position in the hostlist, is 1.

   echo ${MS} > ${HOSTLIST}
   scontrol show hostnames $SLURM_JOB_NODELIST | grep -v ${MS} \
            | uniq | sort >> ${HOSTLIST}

   # Compute the number of nodes assigned.

   export NODES=`wc -l ${HOSTLIST} |gawk '//{print $1}'`
   
   # Make a local copy of the SLURM script since only the mother superior
   # can see it at job start.

   JOBFILE=${WORKDIR}/slurm.${JOBNUM}
   cp $0 $JOBFILE
   chmod u+x ${JOBFILE}

   SLURM_WALLTIME=`gawk '/^#SBATCH --time/{split($2,t,"="); print t[2];}' $0`

   # Mother superior must start up the dispatcher, so:

   python ${WORKDIR}/wq.py --dispatcher ${TASK} --start $START \
       $VERBOSE --time ${SLURM_WALLTIME} --inputs ${FILES} \
       --allworkers $(( $WPN * $NODES )) --jobid ${JOBNUM} &

   # Give it a chance to spin up since the dispatcher must be ready
   # to accept connections from the workers upon request.

   sleep 5

   # Ready to start the script on all compute nodes. This will fire up
   # workers. We'll pass the job number as an argument. They'll
   # connect to the dispatcher and start work immediately.

   for H in `cat ${HOSTLIST}` ; do
      if [ ${H} != ${MS} ] ; then
         ssh -n ${H} ${JOBFILE} ${JOBNUM} &
      fi
   done

   # Finally, mother superior can also start workers:

   python ${WORKDIR}/wq.py --workers ${WPN} --mothersuperior ${MS}

   # Make sure to wait until all the processes are done!

   wait

else

   # Must be running on a compute node. The job number was passed by the
   # mother superior (see above).

   HOSTLIST=${WORKDIR}/hostlist.$1

   # Now, we have to get the name of mother superior from the host
   # list. Thats so we know where the dispatcher is running. Simply
   # grab the first entry from the hostlist file and press on.

   MS=`head -1 ${HOSTLIST}`

   # Ready to go. Spin up the workers. The mother superior passed
   # the job wall time as argument 1 when the script is called, so
   # we have all the values needed for workers:

   python ${WORKDIR}/wq.py --workers ${WPN} --mothersuperior ${MS}

fi

# Give a bit of time to make sure dispatcher has shut down cleanly.
# The WAIT above should allow for this, but coming down on the side of
# paranoia:

sleep 2
