#! /bin/bash
#
# This script does nothing except provide a template for a WQ
# task command script. Since the command line is formed as a 
# string, anything is possible. Two environment variables are
# also provided for the script:
#
#   WQ_CPT ...... cores per task, which can be used for MPI process
#                 or OpenMP thread count. Under PBS, this is computed
#                 as ppn/WPT in wq.pbs.
#   WQ_NUMACTL .. numactl command to control cpu core affinity.
#
# For instance, if the argument is an absolute path to an input file,
# then one could extract the directory path part, and the basename part
# (file name less extension) to use in execution setup:

FILE=$1
DIR=`dirname ${FILE}`
BASE=`basename ${FILE}`

# Other shell magic could come here, like setting up for a small
# MPI run. For OpenMPI, we'd need to build a comma separated list of
# host names (no spaces!), with one name per process desired. Use
# WQ_CPT to determine how many processes the task should use:

HOSTNAME=`uname -n`
HOSTLIST=""
for i in `seq 1 ${WQ_CPT}`; do
   HOSTLIST="${HOSTNAME},${HOSTLIST}"
done 
HOSTLIST=${HOSTLIST%,*}

# Here we create the command line as a string to either display or
# execute later. This is only one of a miriad possibilities, but
# if the command is non-serial, it should start off with the numactl
# string to control cpu affinity. "mb" is the application to run and
# is assumed to be in PATH. If local, use "./mb" or any other legal
# path specifier.

CMD="${WQ_NUMACTL} mpirun"
CMD="${CMD} -host ${HOSTLIST} -np ${WQ_CPT} mb ${FILE} > ${BASE}.out"

# If we want the output of the run to be in the same directory as the
# input file, cd to the directory determined above.

cd $DIR

# For testing purposes, use "if false", and each call will generate
# the command line string, a message about the nodes assigned, and
# then sleep for some time. For actual production, use "if true", and 
# the command will be executed instead. Of course, any other
# processing could be done as well. This is basically just another
# shell script, so adjust to your hearts content. Note that since
# eval is used here, spaces in the CMD string are assumed to seperate
# command line arguments.

if false ; then
   eval "${CMD}"
else
   echo "${CMD}"
   echo "Faking it on HOSTLIST=${HOSTLIST}"
   sleep 2
fi
